# -*- perl -*-
#
# $Id: Nessus.pm,v 1.2 1999/01/29 20:15:42 joe Exp $
#

package Bundle::Net::Nessus;

$VERSION = '0.02';

1;

__END__

=head1 NAME

  Bundle::Net::Nessus - A bundle to install the Net::Nessus package

=head1 SYNOPSIS

  C<perl -MCPAN -e 'install Bundle::Net::Nessus'>

=head1 CONTENTS

Data::Dumper

Term::ReadKey

Net::Telnet 3.01

Net::Nessus

=head1 DESCRIPTION

This bundle includes all that's needed to run the Net::Nessus,
a module for building Nessus clients.

=cut
